"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const transactionVerification_1 = require("./services/transactionVerification");
const _startingMessage = `@shardeum-foundation/archiver starting at
  locale:  ${new Date().toLocaleString()}
  ISO/UTC: ${new Date().toISOString()}`;
console.log(_startingMessage);
console.error(_startingMessage);
const path_1 = require("path");
const fastify_1 = require("fastify");
const cors_1 = require("@fastify/cors");
const rate_limit_1 = require("@fastify/rate-limit");
const clusterModule = require("cluster");
const Config_1 = require("./Config");
const Crypto = require("./Crypto");
const State = require("./State");
const NodeList = require("./NodeList");
const Storage = require("./archivedCycle/Storage");
const Data = require("./Data/Data");
const Cycles = require("./Data/Cycles");
const DataLogWriter_1 = require("./Data/DataLogWriter");
const Utils = require("./Utils");
const StateMetaData_1 = require("./archivedCycle/StateMetaData");
const Logger = require("./Logger");
const fs_1 = require("fs");
const path_2 = require("path");
const memoryReporting_1 = require("./profiler/memoryReporting");
const nestedCounters_1 = require("./profiler/nestedCounters");
const profiler_1 = require("./profiler/profiler");
const statistics_1 = require("./statistics");
const dbstore = require("./dbstore");
const CycleDB = require("./dbstore/cycles");
const ReceiptDB = require("./dbstore/receipts");
const OriginalTxDB = require("./dbstore/originalTxsData");
const saveConsoleOutput_1 = require("./saveConsoleOutput");
const lib_archiver_discovery_1 = require("@shardeum-foundation/lib-archiver-discovery");
const Collector = require("./Data/Collector");
const GlobalAccount_1 = require("./GlobalAccount");
const Cycles_1 = require("./Data/Cycles");
const API_1 = require("./API");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const healthCheck_1 = require("./routes/healthCheck");
const tickets_1 = require("./routes/tickets");
const Helpers_1 = require("./types/ajv/Helpers");
const SchemaHelpers_1 = require("./utils/serialization/SchemaHelpers");
const allowedArchiversManager_1 = require("./shardeum/allowedArchiversManager");
const configFile = (0, path_2.resolve)(__dirname, '../archiver-config.json');
const allowedArchiversConfigPath = (0, path_1.join)(__dirname, '../allowed-archivers.json');
let logDir;
const cluster = clusterModule;
async function start() {
    (0, Config_1.overrideDefaultConfig)(configFile);
    (0, Helpers_1.initAjvSchemas)();
    (0, SchemaHelpers_1.initializeSerialization)();
    // Set crypto hash keys from config
    const hashKey = Config_1.config.ARCHIVER_HASH_KEY;
    Crypto.setCryptoHashKey(hashKey);
    let logsConfig;
    try {
        logsConfig = lib_types_1.Utils.safeJsonParse((0, fs_1.readFileSync)((0, path_2.resolve)(__dirname, '../archiver-log.json'), 'utf8'));
    }
    catch (err) {
        console.log('Failed to parse archiver log file:', err);
    }
    logDir = `${Config_1.config.ARCHIVER_LOGS}/${Config_1.config.ARCHIVER_IP}_${Config_1.config.ARCHIVER_PORT}`;
    const baseDir = '.';
    logsConfig.dir = logDir;
    Logger.initLogger(baseDir, logsConfig);
    if (logsConfig.saveConsoleOutput) {
        (0, saveConsoleOutput_1.startSaving)((0, path_1.join)(baseDir, logsConfig.dir));
    }
    // Initialize allowed archivers manager
    allowedArchiversManager_1.allowedArchiversManager.initialize(allowedArchiversConfigPath);
    // Global error handling
    process.on('uncaughtException', (error) => {
        Logger.mainLogger.error('Uncaught Exception - Global:', error);
    });
    process.on('unhandledRejection', (reason, promise) => {
        Logger.mainLogger.error('Unhandled Rejection - Global:', promise, 'reason:', reason);
    });
    // Initialize storage
    if (Config_1.config.experimentalSnapshot) {
        await dbstore.initializeDB(Config_1.config);
    }
    else {
        await Storage.initStorage(Config_1.config);
    }
    State.addSigListeners();
    if (!cluster.isPrimary) {
        // Initialize state from config
        await State.initFromConfig(Config_1.config, false, false);
        // await initWorkerProcess()
        return;
    }
    const lastStoredCycle = await CycleDB.queryLatestCycleRecords(1);
    if (lastStoredCycle && lastStoredCycle.length > 0) {
        const lastStoredCycleMode = lastStoredCycle[0].mode;
        if (lastStoredCycleMode === 'shutdown') {
            (0, Cycles_1.setShutdownCycleRecord)(lastStoredCycle[0]);
            Logger.mainLogger.debug('Found shutdown cycleRecord', Cycles_1.cycleRecordWithShutDownMode);
            // Initialize state from config
            await State.initFromConfig(Config_1.config, true);
            const result = await State.compareCycleRecordWithOtherArchivers(Cycles_1.cycleRecordWithShutDownMode.archiversAtShutdown, Cycles_1.cycleRecordWithShutDownMode);
            if (result) {
                State.resetActiveArchivers(Cycles_1.cycleRecordWithShutDownMode.archiversAtShutdown);
                // Load global account from db
                await (0, GlobalAccount_1.loadGlobalAccounts)();
                await startServer();
                return;
            }
        }
    }
    try {
        await (0, lib_archiver_discovery_1.setupArchiverDiscovery)({
            hashKey,
            customConfigPath: configFile.toString(),
            archiverTimeoutInMilliSeconds: 2000, // 2 seconds
        });
    }
    catch (e) {
        console.log('Error setting up archiver discovery: ', e);
    }
    // Initialize state from config
    await State.initFromConfig(Config_1.config);
    if (State.isFirst) {
        Logger.mainLogger.debug('We are first archiver. Starting archive-server');
        const lastStoredCycle = await CycleDB.queryLatestCycleRecords(1);
        if (lastStoredCycle && lastStoredCycle.length > 0) {
            // Load global account from db
            await (0, GlobalAccount_1.loadGlobalAccounts)();
            // Seems you got restarted, and there are no other archivers to check; build nodelists and send join request to the nodes first
            await Data.buildNodeListFromStoredCycle(lastStoredCycle[0]);
            let isJoined = false;
            let firstTime = true;
            const cycleDuration = Cycles.currentCycleDuration;
            do {
                try {
                    // Get active nodes from Archiver
                    const nodeList = NodeList.getActiveList();
                    // try to join the network
                    isJoined = await Data.joinNetwork(nodeList, firstTime);
                }
                catch (err) {
                    Logger.mainLogger.error('Error while joining network:');
                    Logger.mainLogger.error(err);
                    Logger.mainLogger.error(err.stack);
                    Logger.mainLogger.debug(`Trying to join again in ${cycleDuration} seconds...`);
                    await Utils.sleep(cycleDuration);
                }
                firstTime = false;
            } while (!isJoined);
            /**
             * [NOTE] [AS] There's a possibility that we could get stuck in this loop
             * if the joinRequest was sent in the wrong cycle quarter (Q2, Q3, or Q4).
             *
             * Since we've dealt with this problem in shardus-global-server, it might be
             * good to refactor this code to do what shardus-global-server does to join
             * the network.
             */
            Logger.mainLogger.debug('We have successfully joined the network');
            await startServer();
            await Data.subscribeNodeForDataTransfer();
        }
        else {
            await startServer();
        }
    }
    else {
        Logger.mainLogger.debug('We are not first archiver. Syncing and starting archive-server');
        syncAndStartServer();
    }
    setTimeout(() => {
        (0, transactionVerification_1.scheduleMultiSigKeysSyncFromNetConfig)();
    }, 60 * 1000); // Start after 60 seconds
}
function initProfiler(server) {
    const memoryReporter = new memoryReporting_1.default(server);
    (0, memoryReporting_1.setMemoryReportingInstance)(memoryReporter);
    const nestedCounter = new nestedCounters_1.default(server);
    (0, nestedCounters_1.setNestedCountersInstance)(nestedCounter);
    const profiler = new profiler_1.default(server);
    (0, profiler_1.setProfilerInstance)(profiler);
    const statistics = new statistics_1.default(logDir, Config_1.config.STATISTICS, {
        counters: [],
        watchers: {},
        timers: [],
        manualStats: ['cpuPercent'],
    }, {});
    statistics.startSnapshots();
    statistics.on('snapshot', memoryReporting_1.memoryReportingInstance.updateCpuPercent);
    // ========== ENDPOINTS ==========
    memoryReporter.registerEndpoints();
    nestedCounter.registerEndpoints();
    profiler.registerEndpoints();
}
/** Asynchronous function to synchronize and start the server. */
async function syncAndStartServer() {
    // Validate data if there is any in db
    // Retrieve the count of receipts currently stored in the database
    let lastStoredReceiptCount = await ReceiptDB.queryReceiptCount();
    // Retrieve the count of cycles currently stored in the database
    let lastStoredCycleCount = await CycleDB.queryCyleCount();
    let lastStoredOriginalTxCount = await OriginalTxDB.queryOriginalTxDataCount();
    // Query the latest cycle record from the database
    let lastStoredCycleInfo = (await CycleDB.queryLatestCycleRecords(1))[0];
    // Initialize last stored receipt cycle as 0
    let lastStoredReceiptCycle = 0;
    let lastStoredOriginalTxCycle = 0;
    // Request total data from the random archiver
    const response = await Data.getTotalDataFromArchivers();
    // Check if the response is valid and all data fields are non-negative
    if (!response ||
        response.totalCycles < 0 ||
        response.totalAccounts < 0 ||
        response.totalTransactions < 0 ||
        response.totalReceipts < 0) {
        throw Error(`Can't fetch total data from archivers`);
    }
    // Destructure the response to get total counts for cycles, accounts, transactions and receipts
    const { totalCycles, totalReceipts } = response;
    // Check if local database has more data than the network, if so, clear the database
    if (lastStoredReceiptCount > totalReceipts || lastStoredCycleCount > totalCycles) {
        throw Error('The existing db has more data than the network data! Clear the DB and start the server again!');
    }
    // If there are stored cycles, validate the old cycle data
    if (lastStoredCycleCount > 0) {
        Logger.mainLogger.debug('Validating old cycles data!');
        // Compare old cycle data with the archiver data
        const cycleResult = await Data.compareWithOldCyclesData(lastStoredCycleInfo.counter);
        // If the cycle data does not match, clear the DB and start again
        if (!cycleResult.success) {
            throw Error('The last saved 10 cycles data does not match with the archiver data! Clear the DB and start the server again!');
        }
        // Update the last stored cycle count
        lastStoredCycleCount = cycleResult.matchedCycle;
    }
    // Log the last stored cycle and receipt counts
    Logger.mainLogger.debug('lastStoredCycleCount', lastStoredCycleCount, 'lastStoredReceiptCount', lastStoredReceiptCount, 'lastStoredOriginalTxCount', lastStoredOriginalTxCount);
    // If your not the first archiver node, get a nodelist from the others
    // Initialize variables for joining the network
    let isJoined = false;
    let firstTime = true;
    // Get the cycle duration
    const cycleDuration = await Data.getCycleDuration();
    // Attempt to join the network until successful
    do {
        try {
            const randomArchiver = State.getRandomArchiver();
            // Get active nodes from Archiver
            const nodeList = await NodeList.getActiveNodeListFromArchiver(randomArchiver);
            // If no nodes are active, retry the loop
            if (nodeList.length === 0)
                continue;
            // Attempt to join the network
            isJoined = await Data.joinNetwork(nodeList, firstTime);
        }
        catch (err) {
            // Log the error if the joining process fails
            Logger.mainLogger.error('Error while joining network:');
            Logger.mainLogger.error(err);
            Logger.mainLogger.error(err.stack);
            // Sleep for a cycle duration and then retry
            Logger.mainLogger.debug(`Trying to join again in ${cycleDuration} seconds...`);
            await Utils.sleep(cycleDuration * 1000);
        }
        // After the first attempt, set firstTime to false
        firstTime = false;
    } while (!isJoined);
    /**
     * [NOTE] [AS] There's a possibility that we could get stuck in this loop
     * if the joinRequest was sent in the wrong cycle quarter (Q2, Q3, or Q4).
     *
     * Since we've dealt with this problem in shardus-global-server, it might be
     * good to refactor this code to do what shardus-global-server does to join
     * the network.
     */
    Logger.mainLogger.debug('We have successfully joined the network');
    // Once the archiver is joined, check if the existing data (receipt/originalTxData) is valid
    // If there are stored receipts, validate the old receipt data
    if (lastStoredReceiptCount > 0) {
        Logger.mainLogger.debug('Validating old receipts data!');
        // Query latest receipts from the DB
        const lastStoredReceiptInfo = await ReceiptDB.queryLatestReceipts(1);
        // If there's any stored receipt, update lastStoredReceiptCycle
        if (lastStoredReceiptInfo && lastStoredReceiptInfo.length > 0)
            lastStoredReceiptCycle = lastStoredReceiptInfo[0].cycle;
        // Compare old receipts data with the archiver data
        const receiptResult = await Data.compareWithOldReceiptsData(lastStoredReceiptCycle);
        // If the receipt data does not match, clear the DB and start again
        if (!receiptResult.success) {
            throw Error('The last saved receipts of last 10 cycles data do not match with the archiver data! Clear the DB and start the server again!');
        }
        // Update the last stored receipt cycle
        lastStoredReceiptCycle = receiptResult.matchedCycle;
    }
    if (lastStoredOriginalTxCount > 0) {
        Logger.mainLogger.debug('Validating old Original Txs data!');
        const lastStoredOriginalTxInfo = await OriginalTxDB.queryLatestOriginalTxs(1);
        if (lastStoredOriginalTxInfo && lastStoredOriginalTxInfo.length > 0)
            lastStoredOriginalTxCycle = lastStoredOriginalTxInfo[0].cycle;
        const txResult = await Data.compareWithOldOriginalTxsData(lastStoredOriginalTxCycle);
        if (!txResult.success) {
            throw Error('The saved Original-Txs of last 10 cycles data do not match with the archiver data! Clear the DB and start the server again!');
        }
        lastStoredOriginalTxCycle = txResult.matchedCycle;
    }
    // Synchronize Genesis accounts and transactions from the network archivers
    await Data.syncGenesisAccountsFromArchiver(); // Sync Genesis Accounts that the network start with.
    await Data.syncGenesisTransactionsFromArchiver();
    // Sync cycle and node list information
    if (Config_1.config.useSyncV2 === true) {
        await Data.syncCyclesAndNodeListV2(State.activeArchivers, lastStoredCycleCount);
    }
    else {
        await Data.syncCyclesAndNodeList(lastStoredCycleCount);
    }
    // If experimentalSnapshot is enabled, perform receipt synchronization
    if (Config_1.config.experimentalSnapshot) {
        // Sync GlobalAccountsList and cache the Global Network Account
        await (0, GlobalAccount_1.syncGlobalAccount)();
        // If no receipts stored, synchronize all receipts, otherwise synchronize by cycle
        if (lastStoredReceiptCount === 0)
            await Data.syncReceipts();
        else {
            Logger.mainLogger.debug('lastStoredReceiptCycle', lastStoredReceiptCycle);
            await Data.syncReceiptsByCycle(lastStoredReceiptCycle);
        }
        if (lastStoredOriginalTxCount === 0)
            await Data.syncOriginalTxs();
        else {
            Logger.mainLogger.debug('lastStoredOriginalTxCycle', lastStoredOriginalTxCycle);
            await Data.syncOriginalTxsByCycle(lastStoredOriginalTxCycle);
        }
        // After receipt data syncing completes, check cycle and receipt again to be sure it's not missing any data
        // Query for the cycle and receipt counts
        lastStoredReceiptCount = await ReceiptDB.queryReceiptCount();
        lastStoredOriginalTxCount = await OriginalTxDB.queryOriginalTxDataCount();
        lastStoredCycleCount = await CycleDB.queryCyleCount();
        lastStoredCycleInfo = (await CycleDB.queryLatestCycleRecords(1))[0];
        // Check for any missing data and perform syncing if necessary
        if (lastStoredCycleCount - 1 !== lastStoredCycleInfo.counter) {
            throw Error(`The archiver has ${lastStoredCycleCount} and the latest stored cycle is ${lastStoredCycleInfo.counter}`);
        }
        await Data.syncCyclesAndTxsData(lastStoredCycleCount, lastStoredReceiptCount, lastStoredOriginalTxCount);
    }
    else {
        // Sync all state metadata until no older data is fetched from other archivers
        await (0, StateMetaData_1.syncStateMetaData)(State.activeArchivers);
    }
    // Wait for one cycle before sending data request if experimentalSnapshot is not enabled
    if (!Config_1.config.experimentalSnapshot)
        await Utils.sleep(cycleDuration * 1000);
    // Start the server
    await startServer();
    if (!Config_1.config.sendActiveMessage) {
        await Data.subscribeNodeForDataTransfer();
        return;
    }
    const beforeCycle = Cycles.getCurrentCycleCounter();
    // Sending active message to the network
    let isActive = false;
    while (!isActive) {
        await Data.sendActiveRequest();
        // TODO not used for now
        // isActive = await Data.checkActiveStatus()
        // Set as true for now, This needs to be removed after the active record for the archiver is added on the validator side
        isActive = true;
    }
    Data.subscribeNodeForDataTransfer();
    // Sync the missing data during the cycle of sending active request
    const latestCycle = await Cycles.getNewestCycleFromArchivers();
    await Data.syncCyclesAndTxsDataBetweenCycles(beforeCycle - 1, latestCycle.counter + 1);
    (0, transactionVerification_1.scheduleMultiSigKeysSyncFromNetConfig)();
}
// Define all endpoints, all requests, and start REST server
async function startServer() {
    const server = (0, fastify_1.default)({
        logger: false,
    });
    await server.register(cors_1.default);
    await server.register(rate_limit_1.default, {
        global: true,
        max: Config_1.config.RATE_LIMIT,
        timeWindow: 10,
        allowList: ['127.0.0.1', '0.0.0.0'], // Excludes local IPs from rate limits
    });
    await server.register(healthCheck_1.healthCheckRouter);
    server.addContentTypeParser('application/json', { parseAs: 'string' }, (req, body, done) => {
        try {
            const jsonString = typeof body === 'string' ? body : body.toString('utf8');
            done(null, lib_types_1.Utils.safeJsonParse(jsonString));
        }
        catch (err) {
            err.statusCode = 400;
            done(err, undefined);
        }
    });
    server.setReplySerializer((payload) => {
        return lib_types_1.Utils.safeStringify(payload);
    });
    initProfiler(server);
    // Initialize the data log writer
    if (Config_1.config.dataLogWrite)
        await (0, DataLogWriter_1.initDataLogWriter)();
    // Register API routes
    (0, API_1.registerRoutes)(server);
    // Start server and bind to port on all interfaces
    server.listen({
        port: Config_1.config.ARCHIVER_PORT,
        host: '0.0.0.0',
    }, (err) => {
        if (err) {
            server.log.error(err);
            process.exit(1);
        }
        Logger.mainLogger.info(`Worker ${process.pid}: Archive-server is listening on http://0.0.0.0:${Config_1.config.ARCHIVER_PORT}`);
        State.setActive();
        Collector.scheduleMissingTxsDataQuery();
        // setupWorkerProcesses(cluster)
    });
}
// Add this before starting the server
try {
    (0, tickets_1.initializeTickets)();
}
catch (err) {
    console.error('Failed to initialize tickets. Server startup aborted:', err);
    process.exit(1);
}
start();
//# sourceMappingURL=server.js.map